#ifndef DISPLAY
#define DISPLAY

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

/*************************************************************/
/********************Einstellungen****************************/
/*************************************************************/

/******Port Definitionen***/
#define SEG_DRIVER_PORT PORTD
#define SEG_DRIVER_DDR DDRD
#define CHAR_DRIVER_PORT PORTC
#define CHAR_DRIVER_DDR DDRC

/******Funktionen**********/
#define DOMASK_INLINE

/******Displayoptionen*****/

//Masken benutzen (werden in der display.c definiert)
//7Segment-Anzeigen Reihenfolge modifizieren
#define SEGMASK
//Segment Reihenfolge modifizieren
#define CHARMASK

//Wunsch-Multiplexfrequenz (Fps)
//wird auf mglichen Timer-Teiler (Prescaler) erhht
#define FREQ 60
#define MINFREQ 60
#define MAXFREQ 1000
//Multiplexfrequenz und Prescaler als Warnung ausgeben
#define SHOW_PRESCALER

//Wenn auskommentiert wird der Buffer nicht erst gelscht, bevor die nchste
//Anzeige angesteuert wird. ->evtl. leichtes Leuchten einiger Segmente
//Bei hoher Multiplexfrequenz strker
#define SAVEMODE
#define USE_FONT
#ifdef USE_FONT

//Font
#include "7SegmentASCII.h"
#define FONT_FUNCTION(a) GetChar(a)

#endif

/*************************************************************/
/**********************Funktionen*****************************/
/*************************************************************/

/* Bei Manchen Funktionen gibt es die Erweiterung _i = Inline-Version
 * Die Erweiterung f (z.b. display_putcf) heit, dass der Font genutzt wird
 */ 

void display_init_i(void);
void display_putc_i(char c, uint8_t pos);
void display_putsx_i(char* c, uint8_t pos);
void display_puts_i(char* c);
void display_clear(void);
void display_test(void);
void display_clear_to(uint8_t value);

void display_init(void);
void display_putc(char c, uint8_t pos);
void display_putsx(char* c, uint8_t pos);
void display_puts(char* c);

//Font Funktionen
#ifdef USE_FONT
void display_putcf_i(char c, uint8_t pos);
void display_putsf_i(char* c);
void display_putsxf_i(char* c, uint8_t pos);

void display_putcf(char c, uint8_t pos);
void display_putsf(char* c);
void display_putsxf(char* c, uint8_t pos);
#endif

//Hilfsroutinen
char DoMask(char value);

/*************************************************************/
/*************************************************************/
/*************************************************************/

#ifndef F_CPU
#define F_CPU 8000000UL
#warning F_CPU wurde mit 8Mhz neudefiniert
#endif
//F_CPU / Takte bis zum Overflow / Anzahl der 7Segment Anzeigen
#define PRESCALER (F_CPU/256/8/FREQ)

#if PRESCALER > 1023
#if PRESCALER != 1024
#undef PRESCALER
#define PRESCALER 1024
#warning Multiplexfrequenz auf F_CPU/2097152 gesetzt, Prescaler = 1024
#else
#if SHOW_PRESCALER
#warning Prescaler = 1024
#endif
#endif
#define PRESCALER_VALUE 5
#else
#if PRESCALER > 255
#if PRESCALER != 256
#undef PRESCALER
#define PRESCALER 256
#warning Multiplexfrequenz auf F_CPU/524288 gesetzt, Prescaler = 256
#else
#if SHOW_PRESCALER
#warning Prescaler = 256
#endif
#endif
#define PRESCALER_VALUE 4
#else
#if PRESCALER > 63
#if PRESCALER != 64
#undef PRESCALER
#define PRESCALER 64
#warning Multiplexfrequenz auf F_CPU/131072 gesetzt, Prescaler = 64
#else
#if SHOW_PRESCALER
#warning Prescaler = 8
#endif
#endif
#define PRESCALER_VALUE 3
#else
#if PRESCALER > 7
#if PRESCALER != 8
#undef PRESCALER
#define PRESCALER 8
#warning Multiplexfrequenz auf F_CPU/16384 gesetzt, Prescaler = 8
#else
#if SHOW_PRESCALER
#warning Prescaler = 8
#endif
#endif
#define PRESCALER_VALUE 2
#else
#if PRESCALER != 1
#undef PRESCALER
#define PRESCALER 1
#warning Multiplexfrequenz auf F_CPU/2048 gesetzt, Kein Prescaler
#warning Maximale Multiplexfrequenz erreicht
#else
#if SHOW_PRESCALER
#warning Kein Prescaler
#endif
#endif
#define PRESCALER_VALUE 1
#endif
#endif
#endif
#endif

#ifndef PRESCALER_VALUE
#error PRESCALER_VALUE ist nicht definiert
#endif

#undef FREQ
#define FREQ F_CPU/256/8/PRESCALER
#if (FREQ)<(MINFREQ)
#error FREQ ist unter der Minimal-Frequenz
#endif
#if (FREQ)>(MAXFREQ)
#error FREQ ist ber der Maximal-Frequenz
#endif

#endif
